import { system } from "@minecraft/server";

export function clearSkyMagicSpell(player) {
  player.playSound("block.enchanting_table.use", { pitch: 1.3, location: player.location, volume: 1.0 });
  player.dimension.spawnParticle("nicothekid:magic_spell_cast_particle", { x: player.location.x, y: player.location.y + 0.5, z: player.location.z });
  player.runCommandAsync(`weather clear`);
}

export function rainMagicSpell(player) {
  player.playSound("block.enchanting_table.use", { pitch: 1.3, location: player.location, volume: 1.0 });
  player.dimension.spawnParticle("nicothekid:magic_spell_cast_particle", { x: player.location.x, y: player.location.y + 0.5, z: player.location.z });
  player.runCommandAsync(`weather rain`);
}

export function thunderstormMagicSpell(player) {
  player.playSound("block.enchanting_table.use", { pitch: 1.3, location: player.location, volume: 1.0 });
  player.dimension.spawnParticle("nicothekid:magic_spell_cast_particle", { x: player.location.x, y: player.location.y + 0.5, z: player.location.z });
  player.runCommandAsync(`weather thunder`);
}

export function hoardingMagicSpell1(player) {
  player.playSound("block.enchanting_table.use", { pitch: 1.3, location: player.location, volume: 1.0 });
  player.dimension.spawnParticle("nicothekid:hoarding_magic_spell_particle_1", { x: player.location.x, y: player.location.y + 0.1, z: player.location.z });
  player.dimension.spawnParticle("nicothekid:hoarding_magic_spell_teleport_particle", { x: player.location.x, y: player.location.y + 0.1, z: player.location.z });
  const validMobs = player.dimension.getEntities({
    excludeFamilies: [ "mob", "inanimate" ],
    types: [ "item" ],
    excludeTags: [ "nicothekid:antimagic" ],
    location: player.location,
    minDistance: 0,
    maxDistance: 8,
  });
  for (const targetMob of validMobs) {
    const mobLocation = targetMob.location;
    targetMob.dimension.playSound("mob.endermen.portal", mobLocation, { pitch: 1.3, volume: 1.0});
    targetMob.dimension.spawnParticle("nicothekid:hoarding_magic_spell_teleport_particle", { x: mobLocation.x, y: mobLocation.y, z: mobLocation.z });
    targetMob.teleport(player.location);
  }
}
export function hoardingMagicSpell2(player) {
  player.playSound("block.enchanting_table.use", { pitch: 1.3, location: player.location, volume: 1.0 });
  player.dimension.spawnParticle("nicothekid:hoarding_magic_spell_particle_2", { x: player.location.x, y: player.location.y + 0.1, z: player.location.z });
  player.dimension.spawnParticle("nicothekid:hoarding_magic_spell_teleport_particle", { x: player.location.x, y: player.location.y + 0.1, z: player.location.z });
  const validMobs = player.dimension.getEntities({
    excludeFamilies: [ "mob", "inanimate" ],
    types: [ "item" ],
    excludeTags: [ "nicothekid:antimagic" ],
    location: player.location,
    minDistance: 0,
    maxDistance: 16,
  });
  for (const targetMob of validMobs) {
    const mobLocation = targetMob.location;
    targetMob.dimension.playSound("mob.endermen.portal", mobLocation, { pitch: 1.3, volume: 1.0});
    targetMob.dimension.spawnParticle("nicothekid:hoarding_magic_spell_teleport_particle", { x: mobLocation.x, y: mobLocation.y, z: mobLocation.z });
    targetMob.teleport(player.location);
  }
}
export function hoardingMagicSpell3(player) {
  player.playSound("block.enchanting_table.use", { pitch: 1.3, location: player.location, volume: 1.0 });
  player.dimension.spawnParticle("nicothekid:hoarding_magic_spell_particle_3", { x: player.location.x, y: player.location.y + 0.1, z: player.location.z });
  player.dimension.spawnParticle("nicothekid:hoarding_magic_spell_teleport_particle", { x: player.location.x, y: player.location.y + 0.1, z: player.location.z });
  const validMobs = player.dimension.getEntities({
    excludeFamilies: [ "mob", "inanimate" ],
    types: [ "item" ],
    excludeTags: [ "nicothekid:antimagic" ],
    location: player.location,
    minDistance: 0,
    maxDistance: 24,
  });
  for (const targetMob of validMobs) {
    const mobLocation = targetMob.location;
    targetMob.dimension.playSound("mob.endermen.portal", mobLocation, { pitch: 1.3, volume: 1.0});
    targetMob.dimension.spawnParticle("nicothekid:hoarding_magic_spell_teleport_particle", { x: mobLocation.x, y: mobLocation.y, z: mobLocation.z });
    targetMob.teleport(player.location);
  }
}

export function homewardBoundMagicSpell(player) {
  const playerSpawnPoint = player.getSpawnPoint();
  if (playerSpawnPoint === undefined) {
    player.playSound("game.player.attack.nodamage", { pitch: 1.1, location: player.location, volume: 1.0 });
    player.runCommandAsync(`tellraw @p {\"rawtext\":[{\"text\":\"§eYou don't have a set spawn point yet.§r\"}]}`);
  }
  else {
    player.dimension.spawnParticle("nicothekid:homeward_bound_magic_spell_casting_particle", { x: player.location.x, y: player.location.y + 0.1, z: player.location.z });
    player.playSound("portal.portal", { pitch: 0.6, location: player.location, volume: 1.0 });
    system.runTimeout(() => {
      player.playSound("mob.endermen.portal", { pitch: 0.3, location: player.location, volume: 2.0 });
      if (playerSpawnPoint.dimension.id === "minecraft:overworld") {
        player.dimension.spawnParticle("nicothekid:homeward_bound_magic_spell_teleport_particle", { x: player.location.x, y: player.location.y + 0.5, z: player.location.z });
        player.runCommandAsync(`gamerule sendcommandfeedback false`);
        player.runCommandAsync(`execute in overworld run teleport @s ${player.getSpawnPoint().x} ${player.getSpawnPoint().y} ${player.getSpawnPoint().z} ~ ~`);
        player.runCommandAsync(`particle nicothekid:homeward_bound_magic_spell_teleport_particle ~ ~ ~`);
        player.runCommandAsync(`gamerule sendcommandfeedback true`);
      }
      if (playerSpawnPoint.dimension.id === "minecraft:nether") {
        player.dimension.spawnParticle("nicothekid:homeward_bound_magic_spell_teleport_particle", { x: player.location.x, y: player.location.y + 0.5, z: player.location.z });
        player.runCommandAsync(`gamerule sendcommandfeedback false`);
        player.runCommandAsync(`execute in nether run teleport @s ${player.getSpawnPoint().x} ${player.getSpawnPoint().y} ${player.getSpawnPoint().z} ~ ~`);
        player.runCommandAsync(`particle nicothekid:homeward_bound_magic_spell_teleport_particle ~ ~ ~`);
        player.runCommandAsync(`gamerule sendcommandfeedback true`);
      }
      if (playerSpawnPoint.dimension.id === "minecraft:the_end") {
        player.dimension.spawnParticle("nicothekid:homeward_bound_magic_spell_teleport_particle", { x: player.location.x, y: player.location.y + 0.5, z: player.location.z });
        player.runCommandAsync(`gamerule sendcommandfeedback false`);
        player.runCommandAsync(`execute in the_end run teleport @s ${player.getSpawnPoint().x} ${player.getSpawnPoint().y} ${player.getSpawnPoint().z} ~ ~`);
        player.runCommandAsync(`particle nicothekid:homeward_bound_magic_spell_teleport_particle ~ ~ ~`);
        player.runCommandAsync(`gamerule sendcommandfeedback true`);
      }
    }, 100);
  }
}
