import { world, system, BlockPermutation, GameMode } from "@minecraft/server";
import { instantDiggableBlocks, magicWands, magicScrolls, userMagicScrolls } from "../../ntk_listing.js";
import { damageItemDurability, decreaseItemStack } from "../../ntk_functions.js";
import { unknownMagicSpell } from "./unknown_scroll.js";
import {
  clearSkyMagicSpell,
  rainMagicSpell,
  thunderstormMagicSpell,
  hoardingMagicSpell1,
  hoardingMagicSpell2,
  hoardingMagicSpell3,
  homewardBoundMagicSpell
} from "./magic_spells.js";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.itemComponentRegistry.registerCustomComponent("nicothekid:magic_wand_on_hit_entity", {
    onHitEntity: eventData => {
      const damager = eventData.attackingEntity;
      const target = eventData.hitEntity;
      const itemUsed = eventData.itemStack;
      if (target === undefined || itemUsed === undefined) return;
      const { x, y, z } = target.location;
      const damagerEquippable = damager.getComponent("equippable");
      
      if (magicWands.includes(itemUsed.typeId)) {
        if (target.matches({ excludeFamilies: [ "spirit" ] })) {
          // Durabilty Damage:
          damageItemDurability(damager, itemUsed, 2, "Mainhand");
        };
      }
      else return;
    }
  });
  initEvent.itemComponentRegistry.registerCustomComponent("nicothekid:magic_wand_on_mine_block", {
    onMineBlock: eventData => {
      const player = eventData.source;
      const itemUsed = eventData.itemStack;
      const block = eventData.block;
      const blockPermutation = eventData.minedBlockPermutation;
      const playerEquippable = player.getComponent("equippable");
      
      if (magicWands.includes(itemUsed.typeId)) {
        if (!instantDiggableBlocks.includes(blockPermutation.type.id) && !blockPermutation.hasTag("nicothekid:instant_diggable")) {
          damageItemDurability(player, itemUsed, 2, "Mainhand");
        }
        else return;
      }
      else return;
    }
  });
  initEvent.itemComponentRegistry.registerCustomComponent("nicothekid:magic_wand_on_use", {
    onUse: eventData => {
      const player = eventData.source;
      const itemUsed = eventData.itemStack;
      const playerEquippable = player.getComponent("equippable");
      const offhandItem = playerEquippable.getEquipment("Offhand");
      
      if (magicWands.includes(itemUsed.typeId)) {
        if (offhandItem === undefined || !magicScrolls.includes(offhandItem.typeId)) {
          player.playSound("game.player.attack.nodamage", { pitch: 1.1, location: player.location, volume: 1.0 });
          player.runCommandAsync(`tellraw @p {\"rawtext\":[{\"text\":\"§eYou don't have any magic scroll in your off-hand to cast a magic spell.§r\"}]}`);
        }
        else return;
      }
      else return;
    }
  });
  initEvent.itemComponentRegistry.registerCustomComponent("nicothekid:magic_wand_on_complete_use", {
    onCompleteUse: eventData => {
      const player = eventData.source;
      const itemUsed = eventData.itemStack;
      const playerEquippable = player.getComponent("equippable");
      const offhandItem = playerEquippable.getEquipment("Offhand");
      
      if (magicWands.includes(itemUsed.typeId)) {
        if (offhandItem === undefined || !magicScrolls.includes(offhandItem.typeId)) {
          return;
        }
        else if (magicScrolls.includes(offhandItem.typeId)) {
          if (userMagicScrolls.includes(offhandItem.typeId)) {
            if (offhandItem.typeId === "nicothekid:scroll_of_clear_sky") {
              clearSkyMagicSpell(player);
              decreaseItemStack(player, offhandItem, 1, "Offhand");
              damageItemDurability(player, itemUsed, 1, "Mainhand");
            }
            if (offhandItem.typeId === "nicothekid:scroll_of_rain") {
              rainMagicSpell(player);
              decreaseItemStack(player, offhandItem, 1, "Offhand");
              damageItemDurability(player, itemUsed, 1, "Mainhand");
            }
            if (offhandItem.typeId === "nicothekid:scroll_of_thunderstorm") {
              thunderstormMagicSpell(player);
              decreaseItemStack(player, offhandItem, 1, "Offhand");
              damageItemDurability(player, itemUsed, 1, "Mainhand");
            }
            if (offhandItem.typeId === "nicothekid:scroll_of_hoarding_1") {
              hoardingMagicSpell1(player);
              decreaseItemStack(player, offhandItem, 1, "Offhand");
              damageItemDurability(player, itemUsed, 1, "Mainhand");
            }
            if (offhandItem.typeId === "nicothekid:scroll_of_hoarding_2") {
              hoardingMagicSpell2(player);
              decreaseItemStack(player, offhandItem, 1, "Offhand");
              damageItemDurability(player, itemUsed, 1, "Mainhand");
            }
            if (offhandItem.typeId === "nicothekid:scroll_of_hoarding_3") {
              hoardingMagicSpell3(player);
              decreaseItemStack(player, offhandItem, 1, "Offhand");
              damageItemDurability(player, itemUsed, 1, "Mainhand");
            }
            if (offhandItem.typeId === "nicothekid:scroll_of_homeward_bound") {
              homewardBoundMagicSpell(player);
              if (player.getSpawnPoint() !== undefined) {
                decreaseItemStack(player, offhandItem, 1, "Offhand");
                damageItemDurability(player, itemUsed, 1, "Mainhand");
              }
            }
            if (offhandItem.typeId === "nicothekid:unknown_scroll") {
              unknownMagicSpell(player);
              decreaseItemStack(player, offhandItem, 1, "Offhand");
              damageItemDurability(player, itemUsed, 1, "Mainhand");
            }
          }
          else {
            player.playSound("block.enchanting_table.use", { pitch: 1.5, location: player.location, volume: 1.0 });
          }
        }
        else return;
      }
      else return;
    }
  });
});